import pyhf
import pytest

from cabinetry import tabulate


@pytest.mark.parametrize(
    "test_input, expected",
    [
        (("abc", 0), "abc\nbin 1"),
        (("abc", 2), "abc\nbin 3"),
    ],
)
def test__header_name(test_input, expected):
    assert tabulate._header_name(*test_input) == expected
    assert tabulate._header_name("abc", 2, unique=False) == "\nbin 3"


def test__yields_per_bin(example_spec_multibin, example_spec_with_background):
    # multiple channels
    model = pyhf.Workspace(example_spec_multibin).model()
    yields = [[[25.0, 5.0]], [[8.0]]]
    total_stdev = [[5.0, 2.0], [1.0]]
    data = [[35, 8], [10]]

    yield_table = tabulate._yields_per_bin(model, yields, total_stdev, data)
    assert yield_table == [
        {
            "sample": "Signal",
            "region_1\nbin 1": "25.00",
            "region_1\nbin 2": "5.00",
            "region_2\nbin 1": "8.00",
        },
        {
            "sample": "total",
            "region_1\nbin 1": "25.00 \u00B1 5.00",
            "region_1\nbin 2": "5.00 \u00B1 2.00",
            "region_2\nbin 1": "8.00 \u00B1 1.00",
        },
        {
            "sample": "data",
            "region_1\nbin 1": "35.00",
            "region_1\nbin 2": "8.00",
            "region_2\nbin 1": "10.00",
        },
    ]

    # multiple samples
    model = pyhf.Workspace(example_spec_with_background).model()
    yields = [[[150.0], [50.0]]]
    total_stdev = [[8.60]]
    data = [[160]]

    yield_table = tabulate._yields_per_bin(model, yields, total_stdev, data)
    assert yield_table == [
        {"sample": "Background", "Signal Region\nbin 1": "150.00"},
        {"sample": "Signal", "Signal Region\nbin 1": "50.00"},
        {"sample": "total", "Signal Region\nbin 1": "200.00 \u00B1 8.60"},
        {"sample": "data", "Signal Region\nbin 1": "160.00"},
    ]


def test__yields_per_channel(example_spec_multibin, example_spec_with_background):
    # multiple channels
    model = pyhf.Workspace(example_spec_multibin).model()
    yields = [[30], [8.0]]
    total_stdev = [5.39, 1.0]
    data = [43, 10]

    yield_table = tabulate._yields_per_channel(model, yields, total_stdev, data)
    assert yield_table == [
        {
            "sample": "Signal",
            "region_1": "30.00",
            "region_2": "8.00",
        },
        {
            "sample": "total",
            "region_1": "30.00 \u00B1 5.39",
            "region_2": "8.00 \u00B1 1.00",
        },
        {
            "sample": "data",
            "region_1": "43.00",
            "region_2": "10.00",
        },
    ]

    # multiple samples
    model = pyhf.Workspace(example_spec_with_background).model()
    yields = [[150.0, 50]]
    total_stdev = [8.60]
    data = [160]

    yield_table = tabulate._yields_per_channel(model, yields, total_stdev, data)
    assert yield_table == [
        {"sample": "Background", "Signal Region": "150.00"},
        {"sample": "Signal", "Signal Region": "50.00"},
        {"sample": "total", "Signal Region": "200.00 \u00B1 8.60"},
        {"sample": "data", "Signal Region": "160.00"},
    ]



import os

import cabinetry


if __name__ == "__main__":
    # set up customized log formatting
    cabinetry.set_logging()

    # check whether input data exists
    if not os.path.exists("ntuples/"):
        print("run util/create_ntuples.py to create input data")
        raise SystemExit

    # import example config file
    config = cabinetry.configuration.load("config_example.yml")
    cabinetry.configuration.print_overview(config)

    # create template histograms
    cabinetry.template_builder.create_histograms(config, method="uproot")

    # perform histogram post-processing
    cabinetry.template_postprocessor.run(config)

    # visualize systematics templates
    cabinetry.visualize.templates(config)

    # build a workspace and save to file
    workspace_path = "workspaces/example_workspace.json"
    ws = cabinetry.workspace.build(config)
    cabinetry.workspace.save(ws, workspace_path)

    # run a fit
    ws = cabinetry.workspace.load(workspace_path)
    model, data = cabinetry.model_utils.model_and_data(ws)
    fit_results = cabinetry.fit.fit(model, data)

    # visualize pulls and correlation matrix
    cabinetry.visualize.pulls(fit_results)
    cabinetry.visualize.correlation_matrix(fit_results)

    # visualize pre- and post-fit distributions
    cabinetry.visualize.data_MC(model, data, config=config)
    cabinetry.visualize.data_MC(model, data, config=config, fit_results=fit_results)



import logging

from . import configuration  # NOQA
from . import fit  # NOQA
from . import route  # NOQA
from . import smooth  # NOQA
from . import tabulate  # NOQA
from . import template_builder  # NOQA
from . import template_postprocessor  # NOQA
from . import model_utils  # NOQA
from . import visualize  # NOQA
from . import workspace  # NOQA


__version__ = "0.2.2"


def set_logging() -> None:
    """Sets up customized and verbose logging output.

    Logging can be alternatively customized with the Python ``logging`` module directly.
    """
    logging.basicConfig(format="%(levelname)s - %(name)s - %(message)s")
    logging.getLogger("cabinetry").setLevel(logging.DEBUG)
    logging.getLogger("pyhf").setLevel(logging.INFO)
